import os
import time
import datetime
import random
import re
import pandas as pd
import xlrd
import xlsxwriter
import xlwt
from flask import jsonify, request, current_app
from sqlalchemy import or_, and_, desc
from apps.models import *
from apps.util import login_required, verify_token
from apps.inves_manage import api_manage
from apps.utils.response_code import RET
from dateutil.relativedelta import relativedelta


# 构造项目编号序号
def get_num(num):
    if len(str(int(num))) == 1:
        if int(num) == 9:
            num = '0' + str(num + 1)
        else:
            num = '00' + str(num + 1)
    elif len(str(int(num))) == 2:
        if int(num) == 99:
            num = '100'
        else:
            num = '0' + str(num + 1)
    elif len(str(int(num))) == 3:
        num = str(num + 1)
    else:
        num = 'error'
    return num


# 上传线索库
def upload_xiansuo_project(row_content, time_strf, upload_time):
    try:
        project_name = row_content[1]  # 项目名称
        investor_name = row_content[2]  # 投资方名称
        investor_district = row_content[3]  # 投资方所在地
        project_type = row_content[4]  # 项目单位（投资方项目类型
        investment_volume = row_content[5]  # 总投资额(万元)
        if investment_volume:
            investment_volume = float(investment_volume)
        investment_volume1 = row_content[6]  # 拟引资额(万元)
        if investment_volume1:
            investment_volume = float(investment_volume1)
        project_info = row_content[7]  # 项目基本情况
        project_info1 = row_content[8]  # 项目方基本情况
        thread_people = row_content[9]  # 线索提供人
        thread_people_unity = row_content[10]  # 线索提供人单位或职务
        thread_people_mobile = row_content[11]  # 线索提供人联系方式
        remark = row_content[12]  # 备注

        project_obj_list = SikuProject.query.all()  # 项目编号
        max_project_num = max([project_obj.project_num for project_obj in project_obj_list])
        if max_project_num:
            max_project_num = max_project_num[8:]
            num = get_num(max_project_num)
            if num == 'error':
                # return jsonify(code=RET.DATAERR, msg='编号超出三位的规定范围，请联系管理员')
                return 'error'
            project_num = time_strf + num
        else:
            project_num = time_strf + '001'

        # 创建项目
        new_project_obj = SikuProject(
            upload_time=upload_time,
            project_year=upload_time.split('/')[0],
            project_name=project_name,
            project_num=project_num,
            investor_name=investor_name,
            investor_district=investor_district,
            project_type=project_type,
            investment_volume=investment_volume,
            investment_volume1=investment_volume1,
            project_info=project_info,
            project_info1=project_info1,
            thread_people=thread_people,
            thread_people_unity=thread_people_unity,
            thread_people_mobile=thread_people_mobile,
            remark=remark,
        )
        db.session.add(new_project_obj)
        db.session.commit()
        return 'continue'
    except Exception as e:
        return 'error1'


# 更新对接库
def update_joint_project(row_content):
    try:
        project_num = row_content[0]  # 项目名称
        project_name = row_content[1]  # 项目名称
        investor_name = row_content[2]  # 投资方名称
        investor_district1 = row_content[3]  # 投资方所在地,省
        investor_district2 = row_content[4]  # 投资方所在地，市
        investor_district = investor_district1 + '/' + investor_district2

        project_type1 = row_content[5]  # 项目单位（项目类型），一级行业
        project_type2 = row_content[6]  # 项目单位（项目类型），二级行业
        project_type = project_type1 + '/' + project_type2
        investment_volume = row_content[7]  # 总投资额(万元总投资额(万元)
        if investment_volume:
            investment_volume = float(investment_volume)
        investment_volume1 = row_content[8]  # 拟引资额(万元拟引资额(万元)
        if investment_volume1:
            investment_volume = float(investment_volume1)
        project_to_area = row_content[9]  # 拟落地区域
        development_area = row_content[10]  # 开发区
        project_address = row_content[11]  # 项目详细地址
        joint_condition = row_content[12]  # 项目对接情况
        project_problem = row_content[13]  # 项目当前存在的问题
        project_stalker = row_content[14]  # 项目跟踪人
        joint_person = row_content[15]  # 对接人
        joint_person_unity = row_content[16]  # 对接人单位及职务
        joint_person_mobile = row_content[17]  # 对接人联系方式
        project_people = row_content[18]  # 项目方联系人
        project_people_unity = row_content[19]  # 项目方单位或职务
        project_people_mobile = row_content[20]  # 项目方联系方式
        remark = row_content[21]  # 备注

        project_obj = SikuProject.query.filter_by(
            project_num=project_num, project_name=project_name).first()  # 项目编号

        # 更新项目信息
        project_obj.project_name = project_name
        project_obj.investor_name = investor_name
        project_obj.investor_district = investor_district
        project_obj.project_type = project_type
        project_obj.investment_volume = investment_volume
        project_obj.investment_volume1 = investment_volume1
        project_obj.remark = remark
        project_obj.project_to_area = project_to_area
        project_obj.development_area = development_area
        project_obj.project_address = project_address
        project_obj.joint_condition = joint_condition
        project_obj.project_problem = project_problem
        project_obj.project_stalker = project_stalker
        project_obj.joint_person = joint_person
        project_obj.joint_person_unity = joint_person_unity
        project_obj.joint_person_mobile = joint_person_mobile
        project_obj.project_people = project_people
        project_obj.project_people_unity = project_people_unity
        project_obj.project_people_mobile = project_people_mobile
        db.session.commit()
        return 'continue'
    except Exception as e:
        return 'error1'


# 更新签约库
def update_sign_project(row_content, upload_time):
    try:
        project_num = row_content[0]  # 项目编号
        project_name = row_content[1]  # 项目名称
        attract_name = row_content[2]  # 引资方名称
        investor_name = row_content[3]  # 投资方名称
        investor_district1 = row_content[4]  # 投资方所在地,省
        investor_district2 = row_content[5]  # 投资方所在地，市
        investor_district = investor_district1 + '/' + investor_district2

        project_type1 = row_content[6]  # 项目单位（项目类型），一级行业
        project_type2 = row_content[7]  # 项目单位（项目类型），二级行业
        project_type = project_type1 + '/' + project_type2
        investment_volume = row_content[8]  # 总投资额(万元总投资额(万元)
        if investment_volume:
            investment_volume = float(investment_volume)
        investment_volume1 = row_content[9]  # 拟引资额(万元拟引资额(万元)
        if investment_volume1:
            investment_volume = float(investment_volume1)
        project_to_area = row_content[10]  # 拟落地区域
        development_area = row_content[11]  # 开发区
        project_address = row_content[12]  # 项目详细地址

        progress_condition = row_content[13]  # 项目推进情况
        Party_A_name = row_content[14]  # 签约甲方名称
        Party_A_people = row_content[15]  # 签约甲方联系人
        Party_A_mobile = row_content[16]  # 签约甲方联系方式
        Party_B_name = row_content[17]  # 签约乙方名称
        Party_B_people = row_content[18]  # 签约乙方联系人
        Party_B_mobile = row_content[19]  # 签约乙方联系方式
        sign_time = row_content[20]  # 签约时间

        project_stalker = row_content[21]  # 项目跟踪人
        joint_person = row_content[22]  # 对接人
        joint_person_unity = row_content[23]  # 对接人单位及职务
        joint_person_mobile = row_content[24]  # 对接人联系方式
        project_people = row_content[25]  # 项目方联系人
        project_people_unity = row_content[26]  # 项目方单位或职务
        project_people_mobile = row_content[27]  # 项目方联系方式
        remark = row_content[28]  # 备注

        project_obj = SikuProject.query.filter_by(
            project_num=project_num, project_name=project_name).first()  # 项目编号

        # 更新项目信息
        project_obj.project_name = project_name
        project_obj.investor_name = investor_name
        project_obj.attract_name = attract_name
        project_obj.investor_district = investor_district
        project_obj.project_type = project_type
        project_obj.investment_volume = investment_volume
        project_obj.investment_volume1 = investment_volume1
        project_obj.remark = remark
        project_obj.project_to_area = project_to_area
        project_obj.development_area = development_area
        project_obj.project_address = project_address

        project_obj.progress_condition = progress_condition
        project_obj.Party_A_name = Party_A_name
        project_obj.Party_A_people = Party_A_people
        project_obj.Party_A_mobile = Party_A_mobile
        project_obj.Party_B_name = Party_B_name
        project_obj.Party_B_people = Party_B_people
        project_obj.Party_B_mobile = Party_B_mobile
        project_obj.sign_time = sign_time
        project_obj.sign_year = sign_time.splite('/')[0]

        project_obj.project_stalker = project_stalker
        project_obj.joint_person = joint_person
        project_obj.joint_person_unity = joint_person_unity
        project_obj.joint_person_mobile = joint_person_mobile
        project_obj.project_people = project_people
        project_obj.project_people_unity = project_people_unity
        project_obj.project_people_mobile = project_people_mobile
        db.session.commit()
        return 'continue'
    except Exception as e:
        return 'error1'


# 更新开工库
def update_star_project(row_content):
    try:
        project_num = row_content[0]  # 项目编号
        project_name = row_content[1]  # 项目名称
        attract_name = row_content[2]  # 引资方名称
        investor_name = row_content[3]  # 投资方名称
        investor_district1 = row_content[4]  # 投资方所在地,省
        investor_district2 = row_content[5]  # 投资方所在地，市
        investor_district = investor_district1 + '/' + investor_district2

        project_type1 = row_content[6]  # 项目单位（项目类型），一级行业
        project_type2 = row_content[7]  # 项目单位（项目类型），二级行业
        project_type = project_type1 + '/' + project_type2
        investment_volume = row_content[8]  # 总投资额(万元总投资额(万元)
        if investment_volume:
            investment_volume = float(investment_volume)
        investment_volume1 = row_content[9]  # 拟引资额(万元拟引资额(万元)
        if investment_volume1:
            investment_volume = float(investment_volume1)


        investment = row_content[10]  # 到位资金


        project_to_area1 = row_content[11]  # 落地区域
        development_area = row_content[12]  # 开发区
        project_address = row_content[13]  # 项目详细地址

        progress_condition = row_content[14]  # 项目推进情况

        project_problem = row_content[15]  # 项目当前存在的问题
        sign_time = row_content[16]  # 签约时间

        start_time = row_content[17]  # 开工时间
        project_stalker = row_content[18]  # 项目跟踪人
        joint_person = row_content[19]  # 对接人
        joint_person_unity = row_content[20]  # 对接人单位及职务
        joint_person_mobile = row_content[21]  # 对接人联系方式
        project_people = row_content[22]  # 项目方联系人
        project_people_unity = row_content[23]  # 项目方单位或职务
        project_people_mobile = row_content[24]  # 项目方联系方式
        remark = row_content[25]  # 备注

        project_obj = SikuProject.query.filter_by(
            project_num=project_num, project_name=project_name).first()  # 项目编号

        # 更新项目信息
        project_obj.project_num = project_num
        project_obj.project_name = project_name
        project_obj.investor_name = investor_name
        project_obj.attract_name = attract_name
        project_obj.investor_district = investor_district
        project_obj.project_type = project_type
        project_obj.investment_volume = investment_volume
        project_obj.investment_volume1 = investment_volume1
        project_obj.investment = investment

        project_obj.project_to_area1 = project_to_area1
        project_obj.development_area = development_area
        project_obj.project_address = project_address

        project_obj.progress_condition = progress_condition
        project_obj.project_problem = project_problem
        project_obj.sign_time = sign_time
        project_obj.start_time = start_time
        project_obj.start_year = start_time.splite('/')[0]

        project_obj.project_stalker = project_stalker
        project_obj.joint_person = joint_person
        project_obj.joint_person_unity = joint_person_unity
        project_obj.joint_person_mobile = joint_person_mobile
        project_obj.project_people = project_people
        project_obj.project_people_unity = project_people_unity
        project_obj.project_people_mobile = project_people_mobile
        project_obj.remark = remark
        db.session.commit()
        return 'continue'
    except Exception as e:
        return 'error1'


# 线索库搜索接口
@api_manage.route("/SearchProject", methods=["POST"])
def search_project():
    req_dic = request.get_json()
    token = request.headers['token']
    flag = request.headers['flag']  # 1为线索库，2为对接库，3为签约库，4为开工库
    # 线索库
    distribute_condition = req_dic['distribute_condition']  # 分发情况
    project_name = req_dic['project_name']  # 项目名称
    investor_name = req_dic['investor_name']  # 投资方名称
    investor_district = req_dic['investor_district']  # 投资方所在地
    project_type = req_dic['project_type']  # 项目类型
    investment_volume = req_dic['investment_volume']  # 总投资额(万元)
    project_year = req_dic['project_year']  # 项目年份
    thread_progress = req_dic['thread_progress']  # 线索进展
    # 线索库高级筛选
    distribute_condition_list = req_dic['distribute_condition_list']  # 分发情况（列表形式）
    project_type_list = req_dic['project_type_list']  # 项目类型（列表形式）
    thread_progress_list = req_dic['thread_progress_list']  # 线索进展（列表形式）
    investment_volume_list = req_dic['investment_volume_list']  # 总投资额(万元)
    project_year_list = req_dic['project_year_list']  # 项目年份

    # 对接库
    project_to_area = req_dic['project_to_area']  # 拟落地区域
    development_area = req_dic['development_area']  # 开发区
    # 对接库高级筛选
    project_to_area_list = req_dic['project_to_area_list']  # 拟落地区域（列表形式）
    development_area_list = req_dic['development_area_list']  # 开发区（列表形式）

    # 签约库
    project_to_area1 = req_dic['project_to_area1']  # 落地区域
    # 签约库高级筛选
    project_to_area1_list = req_dic['project_to_area1_list']  # 落地区域（列表形式）
    sign_year_list = req_dic['sign_year_list']  # 签约年份区间（列表形式）

    # 开工库高级筛选
    start_year_list = req_dic['start_year_list']  # 开工年份区间（列表形式）

    page = req_dic['page']
    per_page = req_dic['per_page']

    try:
        user_obj = verify_token(token)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.SESSIONERR, msg="进行操作时用户信息校验失败，请重新登陆后尝试！")

    try:
        project_obj = SikuProject.query.filter(
            SikuProject.distribute_condition._in(distribute_condition_list) if distribute_condition_list else SikuProject.distribute_condition == distribute_condition if distribute_condition else text(''),
            SikuProject.project_name == project_name if project_name else text(''),
            SikuProject.investor_name == investor_name if investor_name else text(''),
            SikuProject.investor_district == investor_district if investor_district else text(''),
            SikuProject.project_type._in(project_type_list) if project_type_list else SikuProject.project_type == project_type if project_type else text(''),
            SikuProject.investment_volume > float(investment_volume_list.split('-')[0]) if investment_volume_list else text(''),
            SikuProject.investment_volume < float(investment_volume_list.split('-')[1]) if investment_volume_list else text(''),
            SikuProject.investment_volume > float(investment_volume.split('-')[0]) if investment_volume and not investment_volume_list else text(''),
            SikuProject.investment_volume < float(investment_volume.split('-')[1]) if investment_volume and not investment_volume_list else text(''),
            SikuProject.project_year > project_year_list[0] if project_year_list else text(''),
            SikuProject.project_year < project_year_list[1] if project_year_list else text(''),
            SikuProject.project_year == project_year if project_year and not project_year_list else text(''),
            SikuProject.thread_progress._in(thread_progress_list) if thread_progress_list else SikuProject.thread_progress == thread_progress if thread_progress else text(''),

            # 对接库
            SikuProject.project_to_area._in(
                project_to_area_list) if project_to_area_list else SikuProject.project_to_area == project_to_area if project_to_area else text(''),
            SikuProject.development_area._in(
                development_area_list) if development_area_list else SikuProject.development_area == development_area if development_area else text(''),

            # 签约库
            SikuProject.project_to_area1._in(
                project_to_area1_list) if project_to_area1_list else SikuProject.project_to_area1 == project_to_area1 if project_to_area1 else text(''),
            SikuProject.sign_year > sign_year_list[0] if sign_year_list else text(''),
            SikuProject.sign_year < sign_year_list[1] if sign_year_list else text(''),

            # 开工库高级筛选
            SikuProject.start_year > start_year_list[0] if start_year_list else text(''),
            SikuProject.start_year < start_year_list[1] if start_year_list else text(''),

        )
        if not project_obj.all():
            return jsonify(code=RET.NODATA, msg='没有符合条件的数据！')
        project_size = project_obj.count()

        project_obj_list = project_obj.order_by(SikuProject.upload_time.desc()).paginate(page, per_page).items
        data = ''

        if flag == 1:  # 线索库
            data = [{
                'id': project_obj.id,
                "distribute_condition": project_obj.distribute_condition,  # 分发情况
                "project_name": project_obj.project_name,  # 项目名称
                "investor_name": project_obj.investor_name,  # 投资方名称
                "investor_district": project_obj.investor_district,  # 投资方所在地
                "project_type": project_obj.project_type,  # 项目类型
                "investment_volume": project_obj.investment_volume,  # 总投资额(万元)
                "project_year": project_obj.project_year,  # 项目年份
                "project_info": project_obj.project_info,  # 项目基本情况
                "thread_people": project_obj.thread_people,  # 线索提供人
                "thread_people_mobile": project_obj.thread_people_mobile,  # 线索提供人联系方式
                "thread_progress": project_obj.thread_progress,  # 线索进展
            } for project_obj in project_obj_list]

        if flag == 2:  # 对接库
            data = [{
                'id': project_obj.id,
                "project_name": project_obj.project_name,  # 项目名称
                "investor_name": project_obj.investor_name,  # 投资方名称
                "investor_district": project_obj.investor_district,  # 投资方所在地
                "project_type": project_obj.project_type,  # 项目类型
                "investment_volume": project_obj.investment_volume,  # 总投资额(万元)
                "project_year": project_obj.project_year,  # 项目年份
                "project_schedule": project_obj.project_schedule,  # 项目进展
                "project_to_area": project_obj.project_to_area,  # 拟落地区域
                "development_area": project_obj.development_area,  # 开发区
                "joint_condition": project_obj.joint_condition,  # 项目对接情况
                "project_stalker": project_obj.project_stalker,  # 项目跟踪人
                "joint_person": project_obj.joint_person,  # 对接人
                "joint_person_mobile": project_obj.joint_person_mobile,  # 对接人联系方式
            } for project_obj in project_obj_list]

        if flag == 3:  # 签约库
            data = [{
                'id': project_obj.id,
                "project_name": project_obj.project_name,  # 项目名称
                "investor_name": project_obj.investor_name,  # 投资方名称
                "investor_district": project_obj.investor_district,  # 投资方所在地
                "project_type": project_obj.project_type,  # 项目类型
                "investment_volume": project_obj.investment_volume,  # 总投资额(万元)
                "project_year": project_obj.project_year,  # 项目年份
                "project_schedule": project_obj.project_schedule,  # 项目进展
                "development_area": project_obj.development_area,  # 开发区
                "project_stalker": project_obj.project_stalker,  # 项目跟踪人
                "joint_person": project_obj.joint_person,  # 对接人
                "joint_person_mobile": project_obj.joint_person_mobile,  # 对接人联系方式
                "attract_name": project_obj.attract_name,  # 引资方名称
                "progress_condition": project_obj.progress_condition,  # 项目推进情况
                "sign_time": project_obj.sign_time,  # 签约时间
                "project_to_area1": project_obj.project_to_area1,  # 落地区域
            } for project_obj in project_obj_list]

        if flag == 4:  # 开工库
            data = [{
                'id': project_obj.id,
                "project_name": project_obj.project_name,  # 项目名称
                "investor_name": project_obj.investor_name,  # 投资方名称
                "investor_district": project_obj.investor_district,  # 投资方所在地
                "project_type": project_obj.project_type,  # 项目类型
                "investment_volume": project_obj.investment_volume,  # 总投资额(万元)
                "project_year": project_obj.project_year,  # 项目年份
                "project_schedule": project_obj.project_schedule,  # 项目进展
                "development_area": project_obj.development_area,  # 开发区
                "project_stalker": project_obj.project_stalker,  # 项目跟踪人
                "joint_person": project_obj.joint_person,  # 对接人
                "joint_person_mobile": project_obj.joint_person_mobile,  # 对接人联系方式
                "attract_name": project_obj.attract_name,  # 引资方名称
                "progress_condition": project_obj.progress_condition,  # 项目推进情况
                "sign_year": project_obj.sign_year,  # 签约年份
                "project_to_area1": project_obj.project_to_area1,  # 落地区域
                "start_time": project_obj.start_time,  # 开工时间
            } for project_obj in project_obj_list]
        return jsonify(code=RET.OK, data={"data": data, "size": project_size, "msg": "数据获取成功！"})
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="查询数据库错误！")


# 线索库批量导入
@api_manage.route("/UploadProjectData", methods=["POST"])
def upload_project_data1():
    # token = request.headers['token']
    req_dic = request.get_json()
    data_type = req_dic['data_type']  # 1为线索库，2为对接库，3为签约库，4为开工库
    # try:
    #     user_obj = verify_token(token)
    #     district_name = user_obj.district_name
    #     user_name = user_obj.real_name
    #     user_id = user_obj.id
    #     unit = user_obj.unit
    # except Exception as e:
    #     current_app.logger.error(e)
    #     return jsonify(code=RET.SESSIONERR, msg="进行批量导入操作时用户信息校验失败，请重新登陆后尝试！")

    # 上传的文件
    file = request.files['file']
    upload_time = datetime.now().strftime("%Y/%m/%d")
    time_strf = datetime.now().strftime("%Y%m%d")
    upload_time1 = (datetime.now() + relativedelta(months=2)).strftime("%Y-%m-%d %H:%M:%S")
    year = str(datetime.now().year)

    try:
        f = file.read()  # 文件内容
        data = xlrd.open_workbook(file_contents=f)
        table = data.sheets()[data_type]
        names = data.sheet_names()  # 返回book中所有工作表的名字
        status = data.sheet_loaded(names[0])  # 检查sheet1是否导入完毕
        print(status)
        nrows = table.nrows  # 获取该sheet中的有效行数
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.THIRDERR, msg="读取文件失败！")

    update_num = 0
    old_num = 0
    add_num = 0
    rep_project_li = []
    rep_project_num = 0
    row_list = [table.row_values(i) for i in range(4, nrows)]
    rows = len(row_list)
    total_num = rows
    for i in range(0, rows):
        row_content = row_list[i]
        if not row_content:
            return jsonify(code=RET.NODATA,
                           msg="{}表格内无有效数据！".format('线索库' if data_type == 1 else '对接库' if data_type == 2
                           else '签约库' if data_type == 3 else '开工库' if data_type == 4 else ''))
        if data_type == 1:  # 上传线索库
            # 判断该项目名称是否已经存在
            # project_num = row_content[0]
            project_name = row_content[1]
            project_obj = SikuProject.query.filter(
                SikuProject.project_name == project_name,
                SikuProject.is_delete == 0,
            ).first()

            # 记录同名的项目名称
            if project_obj and project_name == project_obj.project_name:
                rep_project_num += 1
                continue
            ret = upload_xiansuo_project(row_content, time_strf, upload_time)
            if ret == 'continue':
                add_num += 1
                continue
            elif ret == 'error':
                return jsonify(code=RET.DATAERR, msg='编号超出三位的规定范围，请联系管理员！')
            elif ret == 'error1':
                return jsonify(code=RET.DATAERR, msg='数据入库出错，错误位置为第【{}】行！'.format(i))
            else:
                pass
        if data_type == 2:  # 更新对接库
            ret = update_joint_project(row_content)
            if ret == 'continue':
                update_num += 1
                continue
            elif ret == 'error1':
                return jsonify(code=RET.DATAERR, msg='数据更新出错，错误位置为第【{}】行！'.format(i))
            else:
                pass

        if data_type == 3:  # 更新签约库
            ret = update_sign_project(row_content)
            if ret == 'continue':
                update_num += 1
                continue
            elif ret == 'error1':
                return jsonify(code=RET.DATAERR, msg='数据更新出错，错误位置为第【{}】行！'.format(i))
            else:
                pass
        if data_type == 4:  # 更新开工库
            ret = update_star_project(row_content)
            if ret == 'continue':
                update_num += 1
                continue
            elif ret == 'error1':
                return jsonify(code=RET.DATAERR, msg='数据更新出错，错误位置为第【{}】行！'.format(i))
            else:
                pass
    if data_type == 1 and rep_project_li:
        return jsonify(code=RET.OK,
                       list=rep_project_li,
                       msg='本次导入项目信息共{}个，新增项目信息{}个，其中包含重复名称项目{}个，'
                            '请在以下重复名称项目列表选择相应操作项并提交！'.format(total_num, add_num, rep_project_num))
    else:
        return jsonify(code=RET.OK,
                       list=rep_project_li,
                       msg='本次更新项目信息共{}个。'.format(update_num))


# 下载模板
@api_manage.route("/DownloadTemplate", methods=["GET"])
def download_xlsx():
    # req_dic = request.get_json()
    # token = request.headers['token']
    # action_name = '下载模板'
    # try:
    #     section, user_name, operation_mobile, user_id, role_id, belong_unit = verify_identity(token, action_name)
    # except Exception as e:
    #     current_app.logger.error(e)
    #     return jsonify(code=RET.DBERR, msg="进行{}操作时用户信息校验失败，请重新登陆后尝试！".format(action_name))
    build_template = "http://39.100.39.50:8008/aaa_statics/线索库上传模板.xlsx"
    return build_template


# 批量导出-线索库数据
@api_manage.route("/BatchExportProject", methods=["POST"])  # 批量导出测试
def batch_export_project():
    req_dic = request.get_json()
    token = request.headers['token']

    try:
        user_obj = verify_token(token)
        user_name = user_obj.real_name
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.SESSIONERR, msg="进行批量导入操作时用户信息校验失败，请重新登陆后尝试！")

    select_id_list = req_dic["select_id_list"]
    path = './apps/statics/project_file/'
    # path = 'D:/中研项目代码/new_ty/apps/statics/project_file/'
    pro_path = 'http://39.100.39.50:5051/statics/project_file/'  # 项目路径
    if not select_id_list:
        return jsonify(code=RET.PARAMERR, msg="参数不能为空！")
    target_data_list = []
    for select_id in select_id_list:
        try:
            project_obj = SikuProject.query.filter_by(id=select_id).first()
            # 提取数据，写入xslx表格
            data_list = []
            distribute_condition = project_obj.distribute_condition  # 分发情况
            data_list.append(distribute_condition)
            project_name = project_obj.project_name  # 项目名称
            data_list.append(project_name)
            investor_name = project_obj.investor_name  # 投资方名称
            data_list.append(investor_name)
            investor_district = project_obj.investor_district  # 投资方所在地
            data_list.append(investor_district)
            project_type = project_obj.project_type  # 项目类型
            data_list.append(project_type)
            investment_volume = project_obj.investment_volume  # 总投资额(万元)
            data_list.append(investment_volume)
            project_year = project_obj.project_year  # 项目年份
            data_list.append(project_year)
            project_info = project_obj.project_info  # 项目基本情况
            data_list.append(project_info)
            thread_people = project_obj.thread_people  # 线索提供人
            data_list.append(thread_people)
            thread_people_mobile = project_obj.thread_people_mobile  # 线索提供人联系方式
            data_list.append(thread_people_mobile)
            thread_progress = project_obj.thread_progress  # 线索进展
            data_list.append(thread_progress)
            target_data_list.append(data_list)
            # 记录项目动态
            time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        except Exception as e:
            current_app.logger.error(e)
            return jsonify(code=RET.DBERR, msg="数据库查询错误！")
    try:
        sheet_name_titles = {
            "sheet1": ["项目编号（自动生成）", "项目名称", "投资方名称", "投资方所在地", "项目类型",
                       "总投资额（万元）", "拟引资额（万元）", "项目基本情况", "项目方基本情况", "线索提供人",
                       "单位及职务", "联系方式", "备注",]}

        # 大标题（居中、加粗、字号）
        style = xlwt.XFStyle()
        al = xlwt.Alignment()
        al.horz = 0x02  # 设置水平居中
        al.vert = 0x01  # 设置垂直居中
        style.alignment = al
        # 为样式创建字体
        font = xlwt.Font()
        font.name = 'Times New Roman'  # 字体
        font.bold = True  # 加粗
        font.height = 0x00C8 * 2
        # 设置样式
        style.font = font

        # 表头（居中、加粗）
        style1 = xlwt.XFStyle()
        al1 = xlwt.Alignment()
        al1.horz = 0x02  # 设置水平居中
        al1.vert = 0x01  # 设置垂直居中
        style1.alignment = al1
        # style1.alignment.wrap = 1  # 设置自动换行
        # 为样式创建字体
        font = xlwt.Font()
        font.name = 'Times New Roman'  # 字体
        font.bold = True  # 加粗
        # 设置样式
        style1.font = font
        # 设置边框(THIN实线/DASHED虚线)
        borders = xlwt.Borders()
        borders.left = xlwt.Borders.THIN
        borders.right = xlwt.Borders.THIN
        borders.top = xlwt.Borders.THIN
        borders.bottom = xlwt.Borders.THIN
        style1.borders = borders  # Add Borders to Style

        # 普通内容
        style2 = xlwt.XFStyle()
        al2 = xlwt.Alignment()
        al2.horz = 0x02  # 设置水平居中
        al2.vert = 0x01  # 设置垂直居中
        style2.alignment = al2
        style2.alignment.wrap = 1  # 设置自动换行
        # 设置边框(THIN实线/DASHED虚线)
        borders = xlwt.Borders()
        borders.left = xlwt.Borders.THIN
        borders.right = xlwt.Borders.THIN
        borders.top = xlwt.Borders.THIN
        borders.bottom = xlwt.Borders.THIN
        style2.borders = borders  # Add Borders to Style

        # 需要左对齐的 7建设内容、12最新进展、13需要解决的问题
        style3 = xlwt.XFStyle()
        al3 = xlwt.Alignment()
        al3.horz = 0x01  # 设置左对齐
        al3.vert = 0x01  # 设置垂直居中
        style3.alignment = al3
        style3.alignment.wrap = 1  # 设置自动换行
        # 设置边框(THIN实线/DASHED虚线)
        borders = xlwt.Borders()
        borders.left = xlwt.Borders.THIN
        borders.right = xlwt.Borders.THIN
        borders.top = xlwt.Borders.THIN
        borders.bottom = xlwt.Borders.THIN
        style3.borders = borders  # Add Borders to Style

        # 创建excel
        excel = xlwt.Workbook(encoding='utf-8')
        # 写入表头
        for k, v in sheet_name_titles.items():
            sheet_name = k
            sheet = excel.add_sheet(sheet_name)
            # 写入大标题
            title_name = '线索库'
            # 合并单元格，合并第2行到第4行的第4列到第5列
            sheet.write_merge(0, 0, 0, 17, title_name, style)
            tall_style = xlwt.easyxf('font:height 720')
            sheet.row(0).set_style(tall_style)
            # 写入表头
            head_list = v
            for j in range(len(head_list)):
                sheet.write(1, j, head_list[j], style1)
                # 设置单元格列宽
                if j in [7, 12, 13]:
                    sheet.col(j).width = 50 * 256
                else:
                    sheet.col(j).width = 30 * 256

        """===================我是一条木得感情的分割线============================="""

        # 写入内容
        for index, value_list in enumerate(target_data_list, 2):
            for i, value in enumerate(value_list):
                if i in [7, 12, 13]:
                    sheet.write(index, i, value, style3)
                else:
                    sheet.write(index, i, value, style2)
        time_stamp = '{0:%Y%m%d%H%M%S}'.format(datetime.now())
        file_name = "招商引资项目台账{}.xlsx".format(time_stamp)
        excel.save(path + file_name)
        return jsonify(code=RET.OK, url=pro_path + file_name, msg="导出/下载成功！")
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="批量导出操作失败！")
















































